@@ -107,6 +107,11 @@
 #   ifndef CHECK_TEMPLATE
 #       include <R_ext/BLAS.h>
 #   endif
+#   ifdef USE_FC_LEN_T
+#       define STRING_LEN1 , 1
+#   else
+#       define STRING_LEN1
+#   endif
 #   ifndef USE_COMPLEX
 #       define BLASSCALAR SCALAR
 #   else
@@ -114,6 +119,7 @@
 #   endif
 #else
 #   define BLASSCALAR SCALAR
+#   define STRING_LEN1
 #endif
 
 
